package com.hrsk.cloud.eg.domain.api.channel.guide.http;

import com.hrsk.cloud.eg.domain.api.ApiContext;
import com.hrsk.cloud.eg.domain.api.channel.guide.common.ContentTypeEnum;
import com.hrsk.cloud.eg.domain.api.channel.guide.common.KeyValueObject;
import com.hrsk.cloud.eg.domain.api.channel.guide.common.ValueObject;
import com.hrsk.cloud.eg.domain.api.channel.guide.resolver.payload.ApiPayloadResolverManage;
import com.hrsk.pangu.tool.exception.BizException;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.hrsk.cloud.eg.domain.common.Constants.URL_SEARCH;
import static com.hrsk.cloud.eg.domain.common.Constants.URL_SEARCH_SEPARATOR;
import static com.hrsk.cloud.eg.domain.common.Constants.URL_SEPARATOR;
import static com.hrsk.cloud.eg.domain.common.Constants._EQUAL;

/**
 * @author: bianlanzhou
 * @create: 2024-11-11 14:53
 * @description: http api request
 **/
public class HttpApiRequest {
    /**
     * http api请求配置
     */
    private HttpApiRequestConfig httpApiRequestConfig;
    /**
     * 请求方法
     */
    @Getter
    private String method;
    /**
     * url地址
     */
    @Getter
    private String url;
    /**
     * header数据
     */
    @Getter
    private final Map<String, Object> header = new HashMap<>();
    /**
     * 请求payload数据
     */
    @Getter
    private Object payload;

    /**
     * 构造函数
     * @param httpApiRequestConfig 请求配置
     */
    public HttpApiRequest(HttpApiRequestConfig httpApiRequestConfig) {
        this.httpApiRequestConfig = httpApiRequestConfig;
    }

    /**
     * 请求
     * @param input 入参
     * @return this
     */
    public HttpApiRequest request(Map<String,Object> input){
        ApiContext context = new ApiContext(input);
        url(context);
        header(context);
        body(context);
        return this;
    }

    /**
     * 创建URL
     * @param context 上下文
     */
    private void url(ApiContext context){
        StringBuilder urlSb = new StringBuilder(httpApiRequestConfig.getUrl());
        for(ValueObject valueObject : httpApiRequestConfig.getPath()){
            urlSb.append(URL_SEPARATOR).append(valueObject.compile(context).getRealValue());
        }
        if(!CollectionUtils.isEmpty(httpApiRequestConfig.getSearch())){
            StringBuilder searchSb = new StringBuilder();
            for(KeyValueObject keyValueObject : httpApiRequestConfig.getSearch()){
                searchSb.append(URL_SEARCH_SEPARATOR).append(keyValueObject.getKey()).append(_EQUAL).append(keyValueObject.compile(context).getRealValue());
            }
            urlSb.append(URL_SEARCH).append(searchSb.toString().replaceFirst(URL_SEARCH_SEPARATOR,""));
        }
        method = httpApiRequestConfig.getMethod();
        url = urlSb.toString();
    }

    /**
     * 创建header
     * @param context 上下文
     */
    private void header(ApiContext context){
        for(KeyValueObject keyValueObject : httpApiRequestConfig.getHeader()){
            this.header.put(keyValueObject.getKey(),  keyValueObject.compile(context).getRealValue().toString());
        }
    }

    /**
     * 加载请求体
     * @param context 上下文
     */
    private void body(ApiContext context){
        if(StringUtils.isEmpty(httpApiRequestConfig.getBody())){
            return ;
        }
        if(StringUtils.equals(httpApiRequestConfig.getContentType(), ContentTypeEnum.APPLICATION_JSON.getCode())){
            payload = Objects.toString(
                    ApiPayloadResolverManage.getInstance().resolve(httpApiRequestConfig.getContentType(),
                            httpApiRequestConfig.getBody(),context),"{}");
        }else{
            throw new BizException("不支持的请求体类型!");
        }
    }
}
