package com.hrsk.cloud.eg.domain.api.channel.guide.resolver.payload;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.hrsk.cloud.eg.domain.api.ApiContext;
import com.hrsk.cloud.eg.domain.api.channel.guide.common.ValueObject;
import com.hrsk.cloud.eg.domain.common.GsonUtils;
import com.hrsk.pangu.tool.exception.BizException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

/**
 * @author: bianlanzhou
 * @create: 2024-08-01 15:38
 * @description: json类型payload
 **/
@Slf4j
public class ApiJsonPayloadResolver implements ApiPayloadResolver {
    /**
     * JSON配置最大深度
     */
    private static final Integer MAX_DEEP = 10;
    /**
     * 配置前缀标记
     */
    private static final String CV_PREFIX_SIGN = "<cv>";

    /**
     * 配置后缀标记
     */
    private static final String CV_END_SIGN = "</cv>";

    @Override
    public JsonObject resolve(String jsonTemplate, ApiContext context) {
        JsonElement jsonElement = GsonUtils.fromJson(jsonTemplate, JsonElement.class);
        traverseJson(jsonElement,jsonElement,null,context,0);
        return jsonElement.getAsJsonObject();
    }


    /**
     * 遍历json
     * @param jsonElement json元素
     * @param context 上下文
     * @param deep 深度
     */
    public void traverseJson(JsonElement jsonElement, JsonElement currentElement,String currentKey,ApiContext context,Integer deep) {
        deep++;
        if(deep >= MAX_DEEP){
            throw new BizException(String.format("请求payload的json配置深度达到允许的最大深度[%s]!",MAX_DEEP));
        }
        if (jsonElement.isJsonObject()) {
            JsonObject jsonObject = jsonElement.getAsJsonObject();
            for (String key : jsonObject.keySet()) {
                traverseJson(jsonObject.get(key),jsonObject,key,context,deep);
            }
        } else if (jsonElement.isJsonArray()) {
            JsonArray jsonArray = jsonElement.getAsJsonArray();
            for (JsonElement elem : jsonArray) {
                traverseJson(elem,currentElement,null,context,deep);
            }
        } else {
            JsonPrimitive newElement = parseValueConfigTagToRealValue(jsonElement.getAsJsonPrimitive(),context);
            if(StringUtils.isNotBlank(currentKey)){
                currentElement.getAsJsonObject().add(currentKey,newElement);
            }
        }
    }

    /**
     * 将spel表达式转换成真实值 暂时不使用后续如果有spel表达式可以采用类型推测进行适配
     * @param jsonPrimitive json元素
     */
    private JsonPrimitive parseValueConfigTagToRealValue(JsonPrimitive jsonPrimitive, ApiContext context){
        String valueStr = jsonPrimitive.getAsString();
        if(valueStr.startsWith("<cv>") && valueStr.endsWith("</cv>")){
            valueStr = StringUtils.replaceOnce(valueStr,CV_PREFIX_SIGN,"");
            valueStr = StringUtils.substringBeforeLast(valueStr,CV_END_SIGN);
            try {
                ValueObject valueObject = new Gson().fromJson(valueStr, ValueObject.class);
                Object realValue = valueObject.compile(context).getRealValue();
                if(realValue instanceof Number){
                    return new JsonPrimitive((Number)realValue);
                }else if(realValue instanceof String){
                    return new JsonPrimitive((String)realValue);
                }else if(realValue instanceof Boolean){
                    return new JsonPrimitive((Boolean)realValue);
                }else if(realValue instanceof Character){
                    return new JsonPrimitive((Character)realValue);
                }
            }catch (Exception ex){
                log.error("'{}'解析失败，转为字符串类型。",valueStr,ex);
                return jsonPrimitive;
            }
        }
        return jsonPrimitive;
    }


}
